# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_gateway_error import BadGatewayError
from ..errors.gateway_timeout_error import GatewayTimeoutError
from ..errors.internal_server_error import InternalServerError
from ..errors.method_not_allowed_error import MethodNotAllowedError
from ..errors.not_found_error import NotFoundError
from ..errors.not_implemented_error import NotImplementedError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.drivers_auth_token_create_driver_auth_token_response_body import (
    DriversAuthTokenCreateDriverAuthTokenResponseBody,
)

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawPreviewApIsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create_driver_auth_token(
        self,
        *,
        code: str,
        external_id: typing.Optional[str] = OMIT,
        id: typing.Optional[int] = OMIT,
        username: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[DriversAuthTokenCreateDriverAuthTokenResponseBody]:
        """
        Creates a short-lived auth token for a driver.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Driver Auth Token** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Endpoints in this section are in Preview. These APIs are not functional and are instead for soliciting feedback from our API users on the intended design of this API. Additionally, it is not guaranteed that we will be releasing an endpoint included in this section to production. This means that developers should **NOT** rely on these APIs to build business critical applications

        - Samsara may change the structure of a preview API's interface without versioning or any notice to API users.

        - When an endpoint becomes generally available, it will be announced in the API [changelog](https://developers.samsara.com/changelog).


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        code : str
            Required. Random 12+ character string, used with the auth token to help secure the client from intercepted tokens.

        external_id : typing.Optional[str]
            Optional. External ID of the driver, in the format `key:value` (e.g., `payrollId:ABFS18600`). One of `id`, `externalId`, or `username` is required.

        id : typing.Optional[int]
            Optional. Samsara ID of the driver. One of `id`, `externalId`, or `username` is required.

        username : typing.Optional[str]
            Optional. Username of the driver. This is the login identifier configured when the driver is created. One of `id`, `externalId`, or `username` is required.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DriversAuthTokenCreateDriverAuthTokenResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "preview/fleet/drivers/create-auth-token",
            method="POST",
            json={
                "code": code,
                "externalId": external_id,
                "id": id,
                "username": username,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DriversAuthTokenCreateDriverAuthTokenResponseBody,
                    parse_obj_as(
                        type_=DriversAuthTokenCreateDriverAuthTokenResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def lock_vehicle(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Lock a vehicle. This requires a vehicle gateway with locking capabilities.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Vehicle Lock/Unlock** under the Vehicles category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Endpoints in this section are in Preview. These APIs are not functional and are instead for soliciting feedback from our API users on the intended design of this API. Additionally, it is not guaranteed that we will be releasing an endpoint included in this section to production. This means that developers should **NOT** rely on these APIs to build business critical applications

        - Samsara may change the structure of a preview API's interface without versioning or any notice to API users.

        - When an endpoint becomes generally available, it will be announced in the API [changelog](https://developers.samsara.com/changelog).


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            The ID of the vehicle to lock or unlock. This can be a Samsara internal ID or an external ID in the format `samsara.vin:{VIN}`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"preview/fleet/vehicles/{jsonable_encoder(id)}/lock",
            method="PUT",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def unlock_vehicle(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Unlock a vehicle.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Vehicle Lock/Unlock** under the Vehicles category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Endpoints in this section are in Preview. These APIs are not functional and are instead for soliciting feedback from our API users on the intended design of this API. Additionally, it is not guaranteed that we will be releasing an endpoint included in this section to production. This means that developers should **NOT** rely on these APIs to build business critical applications

        - Samsara may change the structure of a preview API's interface without versioning or any notice to API users.

        - When an endpoint becomes generally available, it will be announced in the API [changelog](https://developers.samsara.com/changelog).


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            The ID of the vehicle to lock or unlock. This can be a Samsara internal ID or an external ID in the format `samsara.vin:{VIN}`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"preview/fleet/vehicles/{jsonable_encoder(id)}/lock",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawPreviewApIsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create_driver_auth_token(
        self,
        *,
        code: str,
        external_id: typing.Optional[str] = OMIT,
        id: typing.Optional[int] = OMIT,
        username: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[DriversAuthTokenCreateDriverAuthTokenResponseBody]:
        """
        Creates a short-lived auth token for a driver.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Driver Auth Token** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Endpoints in this section are in Preview. These APIs are not functional and are instead for soliciting feedback from our API users on the intended design of this API. Additionally, it is not guaranteed that we will be releasing an endpoint included in this section to production. This means that developers should **NOT** rely on these APIs to build business critical applications

        - Samsara may change the structure of a preview API's interface without versioning or any notice to API users.

        - When an endpoint becomes generally available, it will be announced in the API [changelog](https://developers.samsara.com/changelog).


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        code : str
            Required. Random 12+ character string, used with the auth token to help secure the client from intercepted tokens.

        external_id : typing.Optional[str]
            Optional. External ID of the driver, in the format `key:value` (e.g., `payrollId:ABFS18600`). One of `id`, `externalId`, or `username` is required.

        id : typing.Optional[int]
            Optional. Samsara ID of the driver. One of `id`, `externalId`, or `username` is required.

        username : typing.Optional[str]
            Optional. Username of the driver. This is the login identifier configured when the driver is created. One of `id`, `externalId`, or `username` is required.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DriversAuthTokenCreateDriverAuthTokenResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "preview/fleet/drivers/create-auth-token",
            method="POST",
            json={
                "code": code,
                "externalId": external_id,
                "id": id,
                "username": username,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DriversAuthTokenCreateDriverAuthTokenResponseBody,
                    parse_obj_as(
                        type_=DriversAuthTokenCreateDriverAuthTokenResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def lock_vehicle(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Lock a vehicle. This requires a vehicle gateway with locking capabilities.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Vehicle Lock/Unlock** under the Vehicles category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Endpoints in this section are in Preview. These APIs are not functional and are instead for soliciting feedback from our API users on the intended design of this API. Additionally, it is not guaranteed that we will be releasing an endpoint included in this section to production. This means that developers should **NOT** rely on these APIs to build business critical applications

        - Samsara may change the structure of a preview API's interface without versioning or any notice to API users.

        - When an endpoint becomes generally available, it will be announced in the API [changelog](https://developers.samsara.com/changelog).


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            The ID of the vehicle to lock or unlock. This can be a Samsara internal ID or an external ID in the format `samsara.vin:{VIN}`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"preview/fleet/vehicles/{jsonable_encoder(id)}/lock",
            method="PUT",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def unlock_vehicle(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Unlock a vehicle.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Vehicle Lock/Unlock** under the Vehicles category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Endpoints in this section are in Preview. These APIs are not functional and are instead for soliciting feedback from our API users on the intended design of this API. Additionally, it is not guaranteed that we will be releasing an endpoint included in this section to production. This means that developers should **NOT** rely on these APIs to build business critical applications

        - Samsara may change the structure of a preview API's interface without versioning or any notice to API users.

        - When an endpoint becomes generally available, it will be announced in the API [changelog](https://developers.samsara.com/changelog).


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            The ID of the vehicle to lock or unlock. This can be a Samsara internal ID or an external ID in the format `samsara.vin:{VIN}`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"preview/fleet/vehicles/{jsonable_encoder(id)}/lock",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .speeding_interval_response_body import SpeedingIntervalResponseBody
from .trip_asset_response_body import TripAssetResponseBody


class TripSpeedingIntervalsResponseBody(UniversalBaseModel):
    """
    TripSpeedingIntervals
    """

    asset: TripAssetResponseBody
    created_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(
            alias="createdAtTime", description="UTC time the trip was created in Samsara in RFC 3339 format."
        ),
    ]
    driver_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="driverId"),
        pydantic.Field(
            alias="driverId",
            description="The driver ID assigned to the trip. Only returned if includeDriverId is set to true. 'null' if no driver id.",
        ),
    ] = None
    intervals: typing.List[SpeedingIntervalResponseBody] = pydantic.Field()
    """
    List of speeding intervals that belong to the trip. The full list of intervals associated with the trip is always returned, empty if no intervals exist.
    """

    trip_start_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="tripStartTime"),
        pydantic.Field(alias="tripStartTime", description="UTC time the trip started in RFC 3339 format."),
    ]
    updated_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="updatedAtTime"),
        pydantic.Field(
            alias="updatedAtTime",
            description="UTC time the trip was last updated in Samsara in RFC 3339 format. Valid updates are when trip's endTime populates or interval.isDismissed changes value.",
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

DvirSubmittedDeviceTriggerDetailsObjectResponseBodyDvirSubmissionTypesItem = typing.Union[
    typing.Literal["SAFE_NO_DEFECTS", "SAFE_WITH_DEFECTS", "UNSAFE"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .tachograph_vehicle_file import TachographVehicleFile

TachographVehicleFileList = typing.List[TachographVehicleFile]



# This file was auto-generated by Fern from our API Definition.

import typing

AttributeAttributeType = typing.Union[
    typing.Literal["single-select", "multi-select", "text", "freeform-multi-select", "number", "date"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class AlertObjectProductResponseBody(UniversalBaseModel):
    """
    The product associated with the alert
    """

    short_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="shortName"),
        pydantic.Field(alias="shortName", description="The short name associated with the product."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .us_driver_ruleset_override_cycle import UsDriverRulesetOverrideCycle
from .us_driver_ruleset_override_restart import UsDriverRulesetOverrideRestart
from .us_driver_ruleset_override_restbreak import UsDriverRulesetOverrideRestbreak
from .us_driver_ruleset_override_us_state_to_override import UsDriverRulesetOverrideUsStateToOverride


class UsDriverRulesetOverride(UniversalBaseModel):
    """
    US Driver Ruleset override for a given driver. If the driver is operating under a ruleset different from the organization default, the override is used. Updating this value only updates the override setting for this driver. Explicitly setting this field to `null` will delete driver's ruleset override. If the driver does not have an override ruleset set, the response will not include any usDriverRulesetOverride information.
    """

    cycle: UsDriverRulesetOverrideCycle = pydantic.Field()
    """
    The driver's working cycle. Valid values: `USA Property (8/70)`, `USA Property (7/60)`, `USA Passenger (8/70)`, `USA Passenger (7/60)`, `Alaska Property (8/80)`, `Alaska Property (7/70)`, `Alaska Passenger (8/80)`, `Alaska Passenger (7/70)`, `California School/FLV (8/80)`, `California Farm (8/112)`, `California Property (8/80)`, `California Flammable Liquid (8/80)`, `California Passenger (8/80)`, `California Motion Picture (8/80)`, `Florida (8/80)`, `Florida (7/70)`, `Nebraska (8/80)`, `Nebraska (7/70)`, `North Carolina (8/80)`, `North Carolina (7/70)`, `Oklahoma (8/70)`, `Oklahoma (7/60)`, `Oregon (8/80)`, `Oregon (7/70)`, `South Carolina (8/80)`, `South Carolina (7/70)`, `Texas (7/70)`, `Wisconsin (8/80)`, `Wisconsin (7/70)`
    """

    restart: UsDriverRulesetOverrideRestart = pydantic.Field()
    """
    Amount of time necessary for the driver to be resting in order to restart their cycle. Valid values: `34-hour Restart`, `24-hour Restart`, `36-hour Restart`, `72-hour Restart`, `None`.
    """

    restbreak: UsDriverRulesetOverrideRestbreak = pydantic.Field()
    """
    The restbreak required for this driver. Valid values: `Property (off-duty/sleeper)`, `California Mealbreak (off-duty/sleeper)`, `None`.
    """

    us_state_to_override: typing_extensions.Annotated[
        UsDriverRulesetOverrideUsStateToOverride,
        FieldMetadata(alias="usStateToOverride"),
        pydantic.Field(
            alias="usStateToOverride",
            description="The jurisdiction of the ruleset applied to this driver. These are specified by either the ISO 3166-2 postal code for the supported US states, or empty string '' for US Federal Ruleset jurisdiction. Valid values: ``, `AK`, `CA`, `FL`, `NE`, `NC`, `OK`, `OR`, `SC`, `TX`, `WI`.",
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .work_order_item_object_request_body_type import WorkOrderItemObjectRequestBodyType


class WorkOrderItemObjectRequestBody(UniversalBaseModel):
    """
    Work Order Related Item object.
    """

    id: str = pydantic.Field()
    """
    ID of the item.
    """

    type: WorkOrderItemObjectRequestBodyType = pydantic.Field()
    """
    The type of item.  Valid values: `DVIR`, `FAULT`, `FORM`, `ISSUE`, `ITEM_TYPE_UNSPECIFIED`, `MAINTENANCE_PREDICTION_EVENT`, `SCHEDULED_MAINTENANCE`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class PatchShippingDocsResponseBodyResponseBody(UniversalBaseModel):
    """
    Response after successfully updating the LogMetaData object.
    """

    adverse_driving_claimed: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="adverseDrivingClaimed"),
        pydantic.Field(
            alias="adverseDrivingClaimed",
            description="Whether the driver has claimed the [Adverse Driving Exemption](https://kb.samsara.com/hc/en-us/articles/360047336792-Adverse-Driving-Exemption) for this HOS day chart.",
        ),
    ] = None
    big_day_claimed: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="bigDayClaimed"),
        pydantic.Field(
            alias="bigDayClaimed",
            description="Whether the driver has claimed the [Big Day Exemption](https://kb.samsara.com/hc/en-us/articles/360057113891-16-Hour-Short-Haul-Exemption-Big-Day-) for this HOS day chart.",
        ),
    ] = None
    carrier_formatted_address: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="carrierFormattedAddress"),
        pydantic.Field(
            alias="carrierFormattedAddress", description="The address of the carrier used for this HOS chart."
        ),
    ] = None
    carrier_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="carrierName"),
        pydantic.Field(alias="carrierName", description="The name of the carrier used for this HOS chart."),
    ] = None
    carrier_us_dot_number: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="carrierUsDotNumber"),
        pydantic.Field(
            alias="carrierUsDotNumber", description="The US DOT number of the carrier used for this HOS chart."
        ),
    ] = None
    home_terminal_formatted_address: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="homeTerminalFormattedAddress"),
        pydantic.Field(
            alias="homeTerminalFormattedAddress",
            description="The address of the Home Terminal used for this HOS chart.",
        ),
    ] = None
    home_terminal_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="homeTerminalName"),
        pydantic.Field(alias="homeTerminalName", description="The name of the Home Terminal used for this HOS chart."),
    ] = None
    is_certified: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="isCertified"),
        pydantic.Field(alias="isCertified", description="Whether this HOS day chart was certified by the driver."),
    ] = None
    is_us_short_haul_active: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="isUsShortHaulActive"),
        pydantic.Field(
            alias="isUsShortHaulActive",
            description="Whether the driver has the 150 air-mile Short Haul Exemption active for this HOS day chart.",
        ),
    ] = None
    trailer_names: typing_extensions.Annotated[
        typing.Optional[typing.List[str]],
        FieldMetadata(alias="trailerNames"),
        pydantic.Field(
            alias="trailerNames",
            description="List of trailer names associated with the driver for the day. If a trailer was associated with a log through the driver app, the trailer name will be the trailer ID.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .speeding_interval_location_response_response_body import SpeedingIntervalLocationResponseResponseBody
from .speeding_interval_response_body_severity_level import SpeedingIntervalResponseBodySeverityLevel


class SpeedingIntervalResponseBody(UniversalBaseModel):
    """
    Speeding Interval Object
    """

    end_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="endTime"),
        pydantic.Field(alias="endTime", description="UTC time the interval ended in RFC 3339 format."),
    ]
    is_dismissed: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="isDismissed"),
        pydantic.Field(alias="isDismissed", description="Whether the interval is dismissed."),
    ]
    location: SpeedingIntervalLocationResponseResponseBody
    max_speed_kilometers_per_hour: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="maxSpeedKilometersPerHour"),
        pydantic.Field(
            alias="maxSpeedKilometersPerHour", description="The max speed exceeded for the speeding interval."
        ),
    ]
    posted_speed_limit_kilometers_per_hour: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="postedSpeedLimitKilometersPerHour"),
        pydantic.Field(
            alias="postedSpeedLimitKilometersPerHour",
            description="The posted speed limit associated with the speeding interval.",
        ),
    ]
    severity_level: typing_extensions.Annotated[
        SpeedingIntervalResponseBodySeverityLevel,
        FieldMetadata(alias="severityLevel"),
        pydantic.Field(
            alias="severityLevel",
            description="Specifies the severity level of the speeding interval.  Valid values: `light`, `moderate`, `heavy`, `severe`",
        ),
    ]
    start_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="startTime"),
        pydantic.Field(alias="startTime", description="UTC time the interval started in RFC 3339 format."),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .v_1_vehicle_maintenance_j_1939 import V1VehicleMaintenanceJ1939
from .v_1_vehicle_maintenance_passenger import V1VehicleMaintenancePassenger


class V1VehicleMaintenance(UniversalBaseModel):
    """
    Contains any J1939/Passenger engine light warnings and engine faults.
    """

    id: int = pydantic.Field()
    """
    ID of the vehicle.
    """

    j_1939: typing_extensions.Annotated[
        typing.Optional[V1VehicleMaintenanceJ1939], FieldMetadata(alias="j1939"), pydantic.Field(alias="j1939")
    ] = None
    passenger: typing.Optional[V1VehicleMaintenancePassenger] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class RoutesSingleUseAddressObjectResponseBody(UniversalBaseModel):
    """
    This field is used to indicate stops along the route for which an address has not been persisted. This field is mutually exclusive with addressId.
    """

    address: typing.Optional[str] = pydantic.Field(default=None)
    """
    Address of the stop.
    """

    latitude: float = pydantic.Field()
    """
    The latitude of the location
    """

    longitude: float = pydantic.Field()
    """
    The longitude of the location
    """

    radius_meters: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="radiusMeters"),
        pydantic.Field(
            alias="radiusMeters",
            description="The radius in meters for the geofence around this location. Must be a positive value.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .form_submission_pdf_export_response_object_response_body_job_status import (
    FormSubmissionPdfExportResponseObjectResponseBodyJobStatus,
)


class FormSubmissionPdfExportResponseObjectResponseBody(UniversalBaseModel):
    """
    Form Submission PDF export response object.
    """

    completed_at_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="completedAtTime"),
        pydantic.Field(
            alias="completedAtTime",
            description="Time when the PDF export job was completed. Included if 'jobStatus' is 'done'. UTC timestamp in RFC 3339 format.",
        ),
    ] = None
    error_message: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="errorMessage"),
        pydantic.Field(
            alias="errorMessage",
            description="An error message for failed PDF export jobs. Included if 'jobStatus' is 'failed'.",
        ),
    ] = None
    expires_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="expiresAtTime"),
        pydantic.Field(
            alias="expiresAtTime",
            description="Time when the PDF export job expires. After expiration, GET requests for this job will fail and clients must create a new one with another POST request. UTC timestamp in RFC 3339 format.",
        ),
    ]
    id: str = pydantic.Field()
    """
    ID of the form submission being exported.
    """

    job_status: typing_extensions.Annotated[
        FormSubmissionPdfExportResponseObjectResponseBodyJobStatus,
        FieldMetadata(alias="jobStatus"),
        pydantic.Field(
            alias="jobStatus",
            description="Status of the PDF export job.  Valid values: `unknown`, `pending`, `done`, `failed`",
        ),
    ]
    pdf_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="pdfId"),
        pydantic.Field(alias="pdfId", description="Unique ID for the PDF export that is created."),
    ]
    pdf_url: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="pdfUrl"),
        pydantic.Field(
            alias="pdfUrl",
            description="URL to download the PDF file. Expires at time specified in 'pdfUrlExpiresAtTime'. Included if 'jobStatus' is 'done'.",
        ),
    ] = None
    pdf_url_expires_at_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="pdfUrlExpiresAtTime"),
        pydantic.Field(
            alias="pdfUrlExpiresAtTime",
            description="Time when the PDF export's 'pdfUrl' expires. After expiration, clients can retrieve a fresh url with another GET request. UTC timestamp in RFC 3339 format.",
        ),
    ] = None
    requested_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="requestedAtTime"),
        pydantic.Field(
            alias="requestedAtTime",
            description="Time when the PDF export POST request was made. UTC timestamp in RFC 3339 format.",
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .reverse_geo import ReverseGeo
from .time import Time
from .vehicle_location_address import VehicleLocationAddress
from .vehicle_location_heading import VehicleLocationHeading
from .vehicle_location_is_ecu_speed import VehicleLocationIsEcuSpeed
from .vehicle_location_latitude import VehicleLocationLatitude
from .vehicle_location_longitude import VehicleLocationLongitude
from .vehicle_location_speed import VehicleLocationSpeed
from .vehicle_stats_decorations import VehicleStatsDecorations


class VehicleStatsListGps(UniversalBaseModel):
    """
    GPS location data for the vehicle.
    """

    address: typing.Optional[VehicleLocationAddress] = None
    decorations: typing.Optional[VehicleStatsDecorations] = None
    heading_degrees: typing_extensions.Annotated[
        typing.Optional[VehicleLocationHeading],
        FieldMetadata(alias="headingDegrees"),
        pydantic.Field(alias="headingDegrees"),
    ] = None
    is_ecu_speed: typing_extensions.Annotated[
        typing.Optional[VehicleLocationIsEcuSpeed],
        FieldMetadata(alias="isEcuSpeed"),
        pydantic.Field(alias="isEcuSpeed"),
    ] = None
    latitude: VehicleLocationLatitude
    longitude: VehicleLocationLongitude
    reverse_geo: typing_extensions.Annotated[
        typing.Optional[ReverseGeo], FieldMetadata(alias="reverseGeo"), pydantic.Field(alias="reverseGeo")
    ] = None
    speed_miles_per_hour: typing_extensions.Annotated[
        typing.Optional[VehicleLocationSpeed],
        FieldMetadata(alias="speedMilesPerHour"),
        pydantic.Field(alias="speedMilesPerHour"),
    ] = None
    time: Time

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .v_1_trip_response_trips import V1TripResponseTrips


class V1TripResponse(UniversalBaseModel):
    """
    Contains the trips for the vehicle in the requested timeframe. A trip is represented as an object that contains startMs, startLocation, startAddress, startCoordinates, endMs, endLocation, endAddress and endCoordinates. Ongoing trips will be returned with 9223372036854775807 as their endMs.
    """

    trips: typing.Optional[typing.List[V1TripResponseTrips]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .post_fuel_purchase_request_body_price_request_body_currency import (
    PostFuelPurchaseRequestBodyPriceRequestBodyCurrency,
)


class PostFuelPurchaseRequestBodyPriceRequestBody(UniversalBaseModel):
    """
    The price of the fuel transaction in the currency of the country where the transaction occurred.
    """

    amount: str = pydantic.Field()
    """
    The money amount.
    """

    currency: PostFuelPurchaseRequestBodyPriceRequestBodyCurrency = pydantic.Field()
    """
    The currency of money. This is a 3-letter ISO 4217 currency code.  Valid values: `usd`, `gbp`, `cad`, `eur`, `chf`, `mxn`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ForwardCollisionDetectionAlertSettingsObjectResponseBodySensitivity = typing.Union[
    typing.Literal["near", "medium", "far"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class V1SafetyReportHarshEvent(UniversalBaseModel):
    """
    List of harsh events
    """

    harsh_event_type: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="harshEventType"),
        pydantic.Field(alias="harshEventType", description="Sensor generated harsh event type."),
    ] = None
    timestamp_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="timestampMs"),
        pydantic.Field(
            alias="timestampMs", description="Timestamp that the harsh event occurred in Unix milliseconds since epoch"
        ),
    ] = None
    vehicle_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="vehicleId"),
        pydantic.Field(alias="vehicleId", description="Vehicle associated with the harsh event"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class HosDrive(UniversalBaseModel):
    """
    Remaining durations for the HOS driving shift limits.
    """

    drive_remaining_duration_ms: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="driveRemainingDurationMs"),
        pydantic.Field(
            alias="driveRemainingDurationMs",
            description="Remaining driving time the driver has in the current shift in milliseconds. For property-carrying drivers, this is the amount of time the driver can drive before hitting the 11-hour limit.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .inattentive_driving_detection_alert_settings_object_response_body_severity import (
    InattentiveDrivingDetectionAlertSettingsObjectResponseBodySeverity,
)


class InattentiveDrivingDetectionAlertSettingsObjectResponseBody(UniversalBaseModel):
    """
    Enables AI detection of inattentive driving events.
    """

    has_in_cab_audio_alerts_enabled: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="hasInCabAudioAlertsEnabled"),
        pydantic.Field(
            alias="hasInCabAudioAlertsEnabled",
            description="Indicates whether in-cab audio alerts for inattentive driving are turned on.",
        ),
    ] = None
    is_enabled: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="isEnabled"),
        pydantic.Field(
            alias="isEnabled", description="Indicates whether AI event detection for inattentive driving is turned on."
        ),
    ] = None
    severity: typing.Optional[InattentiveDrivingDetectionAlertSettingsObjectResponseBodySeverity] = pydantic.Field(
        default=None
    )
    """
    Severity of inattentive driving events. Options include low (alerts for all events), medium (alerts for medium and high severity events), and high (alerts for high severity events only).  Valid values: `low`, `medium`, `high`
    """

    speeding_threshold_mph: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="speedingThresholdMph"),
        pydantic.Field(alias="speedingThresholdMph", description="Alert when speed is over this many miles per hour."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

DetectionLogSafetyEventObjectResponseBodyInboxFilterReason = typing.Union[
    typing.Literal[
        "overDailyLimit",
        "overHourlyLimit",
        "overTripLimit",
        "belowConfidenceThreshold",
        "belowSeverityThreshold",
        "overEventRateLimit",
        "geofenceFilter",
        "belowNudgeThreshold",
        "belowSpeedThreshold",
        "nighttimeFilter",
        "speedingFilter",
        "inCabAlertOnly",
        "unknown",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.drivers_auth_token_create_driver_auth_token_response_body import (
    DriversAuthTokenCreateDriverAuthTokenResponseBody,
)
from .raw_client import AsyncRawPreviewApIsClient, RawPreviewApIsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class PreviewApIsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPreviewApIsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawPreviewApIsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPreviewApIsClient
        """
        return self._raw_client

    def create_driver_auth_token(
        self,
        *,
        code: str,
        external_id: typing.Optional[str] = OMIT,
        id: typing.Optional[int] = OMIT,
        username: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DriversAuthTokenCreateDriverAuthTokenResponseBody:
        """
        Creates a short-lived auth token for a driver.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Driver Auth Token** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Endpoints in this section are in Preview. These APIs are not functional and are instead for soliciting feedback from our API users on the intended design of this API. Additionally, it is not guaranteed that we will be releasing an endpoint included in this section to production. This means that developers should **NOT** rely on these APIs to build business critical applications

        - Samsara may change the structure of a preview API's interface without versioning or any notice to API users.

        - When an endpoint becomes generally available, it will be announced in the API [changelog](https://developers.samsara.com/changelog).


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        code : str
            Required. Random 12+ character string, used with the auth token to help secure the client from intercepted tokens.

        external_id : typing.Optional[str]
            Optional. External ID of the driver, in the format `key:value` (e.g., `payrollId:ABFS18600`). One of `id`, `externalId`, or `username` is required.

        id : typing.Optional[int]
            Optional. Samsara ID of the driver. One of `id`, `externalId`, or `username` is required.

        username : typing.Optional[str]
            Optional. Username of the driver. This is the login identifier configured when the driver is created. One of `id`, `externalId`, or `username` is required.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriversAuthTokenCreateDriverAuthTokenResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.preview_ap_is.create_driver_auth_token(
            code="dp[gZc1wAigz4uGa0Hh",
        )
        """
        _response = self._raw_client.create_driver_auth_token(
            code=code, external_id=external_id, id=id, username=username, request_options=request_options
        )
        return _response.data

    def lock_vehicle(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Lock a vehicle. This requires a vehicle gateway with locking capabilities.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Vehicle Lock/Unlock** under the Vehicles category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Endpoints in this section are in Preview. These APIs are not functional and are instead for soliciting feedback from our API users on the intended design of this API. Additionally, it is not guaranteed that we will be releasing an endpoint included in this section to production. This means that developers should **NOT** rely on these APIs to build business critical applications

        - Samsara may change the structure of a preview API's interface without versioning or any notice to API users.

        - When an endpoint becomes generally available, it will be announced in the API [changelog](https://developers.samsara.com/changelog).


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            The ID of the vehicle to lock or unlock. This can be a Samsara internal ID or an external ID in the format `samsara.vin:{VIN}`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.preview_ap_is.lock_vehicle(
            id="id",
        )
        """
        _response = self._raw_client.lock_vehicle(id, request_options=request_options)
        return _response.data

    def unlock_vehicle(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Unlock a vehicle.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Vehicle Lock/Unlock** under the Vehicles category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Endpoints in this section are in Preview. These APIs are not functional and are instead for soliciting feedback from our API users on the intended design of this API. Additionally, it is not guaranteed that we will be releasing an endpoint included in this section to production. This means that developers should **NOT** rely on these APIs to build business critical applications

        - Samsara may change the structure of a preview API's interface without versioning or any notice to API users.

        - When an endpoint becomes generally available, it will be announced in the API [changelog](https://developers.samsara.com/changelog).


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            The ID of the vehicle to lock or unlock. This can be a Samsara internal ID or an external ID in the format `samsara.vin:{VIN}`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.preview_ap_is.unlock_vehicle(
            id="id",
        )
        """
        _response = self._raw_client.unlock_vehicle(id, request_options=request_options)
        return _response.data


class AsyncPreviewApIsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPreviewApIsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawPreviewApIsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPreviewApIsClient
        """
        return self._raw_client

    async def create_driver_auth_token(
        self,
        *,
        code: str,
        external_id: typing.Optional[str] = OMIT,
        id: typing.Optional[int] = OMIT,
        username: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DriversAuthTokenCreateDriverAuthTokenResponseBody:
        """
        Creates a short-lived auth token for a driver.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Driver Auth Token** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Endpoints in this section are in Preview. These APIs are not functional and are instead for soliciting feedback from our API users on the intended design of this API. Additionally, it is not guaranteed that we will be releasing an endpoint included in this section to production. This means that developers should **NOT** rely on these APIs to build business critical applications

        - Samsara may change the structure of a preview API's interface without versioning or any notice to API users.

        - When an endpoint becomes generally available, it will be announced in the API [changelog](https://developers.samsara.com/changelog).


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        code : str
            Required. Random 12+ character string, used with the auth token to help secure the client from intercepted tokens.

        external_id : typing.Optional[str]
            Optional. External ID of the driver, in the format `key:value` (e.g., `payrollId:ABFS18600`). One of `id`, `externalId`, or `username` is required.

        id : typing.Optional[int]
            Optional. Samsara ID of the driver. One of `id`, `externalId`, or `username` is required.

        username : typing.Optional[str]
            Optional. Username of the driver. This is the login identifier configured when the driver is created. One of `id`, `externalId`, or `username` is required.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriversAuthTokenCreateDriverAuthTokenResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.preview_ap_is.create_driver_auth_token(
                code="dp[gZc1wAigz4uGa0Hh",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_driver_auth_token(
            code=code, external_id=external_id, id=id, username=username, request_options=request_options
        )
        return _response.data

    async def lock_vehicle(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Lock a vehicle. This requires a vehicle gateway with locking capabilities.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Vehicle Lock/Unlock** under the Vehicles category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Endpoints in this section are in Preview. These APIs are not functional and are instead for soliciting feedback from our API users on the intended design of this API. Additionally, it is not guaranteed that we will be releasing an endpoint included in this section to production. This means that developers should **NOT** rely on these APIs to build business critical applications

        - Samsara may change the structure of a preview API's interface without versioning or any notice to API users.

        - When an endpoint becomes generally available, it will be announced in the API [changelog](https://developers.samsara.com/changelog).


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            The ID of the vehicle to lock or unlock. This can be a Samsara internal ID or an external ID in the format `samsara.vin:{VIN}`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.preview_ap_is.lock_vehicle(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.lock_vehicle(id, request_options=request_options)
        return _response.data

    async def unlock_vehicle(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Unlock a vehicle.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Vehicle Lock/Unlock** under the Vehicles category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Endpoints in this section are in Preview. These APIs are not functional and are instead for soliciting feedback from our API users on the intended design of this API. Additionally, it is not guaranteed that we will be releasing an endpoint included in this section to production. This means that developers should **NOT** rely on these APIs to build business critical applications

        - Samsara may change the structure of a preview API's interface without versioning or any notice to API users.

        - When an endpoint becomes generally available, it will be announced in the API [changelog](https://developers.samsara.com/changelog).


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            The ID of the vehicle to lock or unlock. This can be a Samsara internal ID or an external ID in the format `samsara.vin:{VIN}`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.preview_ap_is.unlock_vehicle(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.unlock_vehicle(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

DefectTypesResponseDataResponseBodySectionType = typing.Union[
    typing.Literal[
        "exteriorFront",
        "exteriorRear",
        "exteriorSideUnderneath",
        "interiorDriverCab",
        "interiorPassengerArea",
        "other",
        "unknown",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        UpdateVehicleRequestAuxInputType1,
        UpdateVehicleRequestAuxInputType10,
        UpdateVehicleRequestAuxInputType11,
        UpdateVehicleRequestAuxInputType12,
        UpdateVehicleRequestAuxInputType13,
        UpdateVehicleRequestAuxInputType2,
        UpdateVehicleRequestAuxInputType3,
        UpdateVehicleRequestAuxInputType4,
        UpdateVehicleRequestAuxInputType5,
        UpdateVehicleRequestAuxInputType6,
        UpdateVehicleRequestAuxInputType7,
        UpdateVehicleRequestAuxInputType8,
        UpdateVehicleRequestAuxInputType9,
        UpdateVehicleRequestHarshAccelerationSettingType,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "UpdateVehicleRequestAuxInputType1": ".types",
    "UpdateVehicleRequestAuxInputType10": ".types",
    "UpdateVehicleRequestAuxInputType11": ".types",
    "UpdateVehicleRequestAuxInputType12": ".types",
    "UpdateVehicleRequestAuxInputType13": ".types",
    "UpdateVehicleRequestAuxInputType2": ".types",
    "UpdateVehicleRequestAuxInputType3": ".types",
    "UpdateVehicleRequestAuxInputType4": ".types",
    "UpdateVehicleRequestAuxInputType5": ".types",
    "UpdateVehicleRequestAuxInputType6": ".types",
    "UpdateVehicleRequestAuxInputType7": ".types",
    "UpdateVehicleRequestAuxInputType8": ".types",
    "UpdateVehicleRequestAuxInputType9": ".types",
    "UpdateVehicleRequestHarshAccelerationSettingType": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "UpdateVehicleRequestAuxInputType1",
    "UpdateVehicleRequestAuxInputType10",
    "UpdateVehicleRequestAuxInputType11",
    "UpdateVehicleRequestAuxInputType12",
    "UpdateVehicleRequestAuxInputType13",
    "UpdateVehicleRequestAuxInputType2",
    "UpdateVehicleRequestAuxInputType3",
    "UpdateVehicleRequestAuxInputType4",
    "UpdateVehicleRequestAuxInputType5",
    "UpdateVehicleRequestAuxInputType6",
    "UpdateVehicleRequestAuxInputType7",
    "UpdateVehicleRequestAuxInputType8",
    "UpdateVehicleRequestAuxInputType9",
    "UpdateVehicleRequestHarshAccelerationSettingType",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.assets_location_link_request_config_object import AssetsLocationLinkRequestConfigObject
from ..types.assets_near_location_link_config_object import AssetsNearLocationLinkConfigObject
from ..types.assets_on_route_link_config_object import AssetsOnRouteLinkConfigObject
from ..types.live_sharing_links_create_live_sharing_link_response_body import (
    LiveSharingLinksCreateLiveSharingLinkResponseBody,
)
from ..types.live_sharing_links_get_live_sharing_links_response_body import (
    LiveSharingLinksGetLiveSharingLinksResponseBody,
)
from ..types.live_sharing_links_update_live_sharing_link_response_body import (
    LiveSharingLinksUpdateLiveSharingLinkResponseBody,
)
from .raw_client import AsyncRawLiveSharingLinksClient, RawLiveSharingLinksClient
from .types.get_live_sharing_links_request_type import GetLiveSharingLinksRequestType
from .types.live_sharing_links_create_live_sharing_link_request_body_type import (
    LiveSharingLinksCreateLiveSharingLinkRequestBodyType,
)

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class LiveSharingLinksClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawLiveSharingLinksClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawLiveSharingLinksClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawLiveSharingLinksClient
        """
        return self._raw_client

    def get_live_sharing_links(
        self,
        *,
        ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        type: typing.Optional[GetLiveSharingLinksRequestType] = None,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LiveSharingLinksGetLiveSharingLinksResponseBody:
        """
        Returns all non-expired Live Sharing Links.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Live Sharing Links** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of Live Share Link IDs

        type : typing.Optional[GetLiveSharingLinksRequestType]
            A filter on the data based on the Live Sharing Link type.  Valid values: `all`, `assetsLocation`, `assetsNearLocation`, `assetsOnRoute`

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 100 objects.

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LiveSharingLinksGetLiveSharingLinksResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.live_sharing_links.get_live_sharing_links()
        """
        _response = self._raw_client.get_live_sharing_links(
            ids=ids, type=type, limit=limit, after=after, request_options=request_options
        )
        return _response.data

    def create_live_sharing_link(
        self,
        *,
        name: str,
        type: LiveSharingLinksCreateLiveSharingLinkRequestBodyType,
        assets_location_link_config: typing.Optional[AssetsLocationLinkRequestConfigObject] = OMIT,
        assets_near_location_link_config: typing.Optional[AssetsNearLocationLinkConfigObject] = OMIT,
        assets_on_route_link_config: typing.Optional[AssetsOnRouteLinkConfigObject] = OMIT,
        description: typing.Optional[str] = OMIT,
        expires_at_time: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LiveSharingLinksCreateLiveSharingLinkResponseBody:
        """
        Create Live Sharing Link.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Live Sharing Links** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        name : str
            Name of the Live Sharing Link.

        type : LiveSharingLinksCreateLiveSharingLinkRequestBodyType
            Type of the Live Sharing Link. This field specifies which one of '<type>LinkConfig' objects will be used to configure the sharing link.  Valid values: `assetsLocation`, `assetsNearLocation`, `assetsOnRoute`

        assets_location_link_config : typing.Optional[AssetsLocationLinkRequestConfigObject]

        assets_near_location_link_config : typing.Optional[AssetsNearLocationLinkConfigObject]

        assets_on_route_link_config : typing.Optional[AssetsOnRouteLinkConfigObject]

        description : typing.Optional[str]
            Description for the Live Sharing Link (not applicable for 'assetsOnRoute' type).

        expires_at_time : typing.Optional[str]
            Date that this link expires in RFC 3339 format. Can't be set in the past. If not provided then link will never expire.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LiveSharingLinksCreateLiveSharingLinkResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.live_sharing_links.create_live_sharing_link(
            name="Example Live Sharing Link name",
            type="assetsLocation",
        )
        """
        _response = self._raw_client.create_live_sharing_link(
            name=name,
            type=type,
            assets_location_link_config=assets_location_link_config,
            assets_near_location_link_config=assets_near_location_link_config,
            assets_on_route_link_config=assets_on_route_link_config,
            description=description,
            expires_at_time=expires_at_time,
            request_options=request_options,
        )
        return _response.data

    def delete_live_sharing_link(self, *, id: str, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete Live Sharing Link.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Live Sharing Links** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            Unique identifier for the Live Sharing Link.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.live_sharing_links.delete_live_sharing_link(
            id="id",
        )
        """
        _response = self._raw_client.delete_live_sharing_link(id=id, request_options=request_options)
        return _response.data

    def update_live_sharing_link(
        self,
        *,
        id: str,
        name: str,
        description: typing.Optional[str] = OMIT,
        expires_at_time: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LiveSharingLinksUpdateLiveSharingLinkResponseBody:
        """
        Update Live Sharing Link.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Live Sharing Links** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            Unique identifier for the Live Sharing Link.

        name : str
            Name of the Live Sharing Link.

        description : typing.Optional[str]
            Description for the Live Sharing Link (not applicable for 'assetsOnRoute' type).

        expires_at_time : typing.Optional[str]
            Date that this link expires in RFC 3339 format. Can't be set in the past. If not provided then link will never expire.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LiveSharingLinksUpdateLiveSharingLinkResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.live_sharing_links.update_live_sharing_link(
            id="id",
            name="Example Live Sharing Link name",
        )
        """
        _response = self._raw_client.update_live_sharing_link(
            id=id, name=name, description=description, expires_at_time=expires_at_time, request_options=request_options
        )
        return _response.data


class AsyncLiveSharingLinksClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawLiveSharingLinksClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawLiveSharingLinksClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawLiveSharingLinksClient
        """
        return self._raw_client

    async def get_live_sharing_links(
        self,
        *,
        ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        type: typing.Optional[GetLiveSharingLinksRequestType] = None,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LiveSharingLinksGetLiveSharingLinksResponseBody:
        """
        Returns all non-expired Live Sharing Links.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Live Sharing Links** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of Live Share Link IDs

        type : typing.Optional[GetLiveSharingLinksRequestType]
            A filter on the data based on the Live Sharing Link type.  Valid values: `all`, `assetsLocation`, `assetsNearLocation`, `assetsOnRoute`

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 100 objects.

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LiveSharingLinksGetLiveSharingLinksResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.live_sharing_links.get_live_sharing_links()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_live_sharing_links(
            ids=ids, type=type, limit=limit, after=after, request_options=request_options
        )
        return _response.data

    async def create_live_sharing_link(
        self,
        *,
        name: str,
        type: LiveSharingLinksCreateLiveSharingLinkRequestBodyType,
        assets_location_link_config: typing.Optional[AssetsLocationLinkRequestConfigObject] = OMIT,
        assets_near_location_link_config: typing.Optional[AssetsNearLocationLinkConfigObject] = OMIT,
        assets_on_route_link_config: typing.Optional[AssetsOnRouteLinkConfigObject] = OMIT,
        description: typing.Optional[str] = OMIT,
        expires_at_time: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LiveSharingLinksCreateLiveSharingLinkResponseBody:
        """
        Create Live Sharing Link.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Live Sharing Links** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        name : str
            Name of the Live Sharing Link.

        type : LiveSharingLinksCreateLiveSharingLinkRequestBodyType
            Type of the Live Sharing Link. This field specifies which one of '<type>LinkConfig' objects will be used to configure the sharing link.  Valid values: `assetsLocation`, `assetsNearLocation`, `assetsOnRoute`

        assets_location_link_config : typing.Optional[AssetsLocationLinkRequestConfigObject]

        assets_near_location_link_config : typing.Optional[AssetsNearLocationLinkConfigObject]

        assets_on_route_link_config : typing.Optional[AssetsOnRouteLinkConfigObject]

        description : typing.Optional[str]
            Description for the Live Sharing Link (not applicable for 'assetsOnRoute' type).

        expires_at_time : typing.Optional[str]
            Date that this link expires in RFC 3339 format. Can't be set in the past. If not provided then link will never expire.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LiveSharingLinksCreateLiveSharingLinkResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.live_sharing_links.create_live_sharing_link(
                name="Example Live Sharing Link name",
                type="assetsLocation",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_live_sharing_link(
            name=name,
            type=type,
            assets_location_link_config=assets_location_link_config,
            assets_near_location_link_config=assets_near_location_link_config,
            assets_on_route_link_config=assets_on_route_link_config,
            description=description,
            expires_at_time=expires_at_time,
            request_options=request_options,
        )
        return _response.data

    async def delete_live_sharing_link(
        self, *, id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Delete Live Sharing Link.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Live Sharing Links** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            Unique identifier for the Live Sharing Link.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.live_sharing_links.delete_live_sharing_link(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_live_sharing_link(id=id, request_options=request_options)
        return _response.data

    async def update_live_sharing_link(
        self,
        *,
        id: str,
        name: str,
        description: typing.Optional[str] = OMIT,
        expires_at_time: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LiveSharingLinksUpdateLiveSharingLinkResponseBody:
        """
        Update Live Sharing Link.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Live Sharing Links** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            Unique identifier for the Live Sharing Link.

        name : str
            Name of the Live Sharing Link.

        description : typing.Optional[str]
            Description for the Live Sharing Link (not applicable for 'assetsOnRoute' type).

        expires_at_time : typing.Optional[str]
            Date that this link expires in RFC 3339 format. Can't be set in the past. If not provided then link will never expire.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LiveSharingLinksUpdateLiveSharingLinkResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.live_sharing_links.update_live_sharing_link(
                id="id",
                name="Example Live Sharing Link name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_live_sharing_link(
            id=id, name=name, description=description, expires_at_time=expires_at_time, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.issues_get_issues_response_body import IssuesGetIssuesResponseBody
from ..types.issues_get_issues_stream_response_body import IssuesGetIssuesStreamResponseBody
from ..types.issues_patch_issue_response_body import IssuesPatchIssueResponseBody
from ..types.patch_issue_request_body_assigned_to_request_body import PatchIssueRequestBodyAssignedToRequestBody
from .raw_client import AsyncRawIssuesClient, RawIssuesClient
from .types.issues_patch_issue_request_body_status import IssuesPatchIssueRequestBodyStatus

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class IssuesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawIssuesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawIssuesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawIssuesClient
        """
        return self._raw_client

    def get_issues(
        self,
        *,
        ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> IssuesGetIssuesResponseBody:
        """
        Returns all issues data for the specified IDs.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Issues** under the Forms category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list containing up to 100 issue IDs to filter on. Can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the issue.

        include : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma separated list of additional fields to include on requested objects. Valid values: `externalIds`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        IssuesGetIssuesResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.issues.get_issues()
        """
        _response = self._raw_client.get_issues(ids=ids, include=include, request_options=request_options)
        return _response.data

    def patch_issue(
        self,
        *,
        id: str,
        assigned_to: typing.Optional[PatchIssueRequestBodyAssignedToRequestBody] = OMIT,
        due_date: typing.Optional[dt.datetime] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        status: typing.Optional[IssuesPatchIssueRequestBodyStatus] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> IssuesPatchIssueResponseBody:
        """
        Updates an instance of an issue.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Issues** under the Forms category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            ID of the issue. Can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the issue.

        assigned_to : typing.Optional[PatchIssueRequestBodyAssignedToRequestBody]

        due_date : typing.Optional[dt.datetime]
            Due date of the issue. UTC timestamp in RFC 3339 format.

        external_ids : typing.Optional[typing.Dict[str, str]]
            A map of external ids

        status : typing.Optional[IssuesPatchIssueRequestBodyStatus]
            Status of the issue.  Valid values: `open`, `inProgress`, `resolved`, `dismissed`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        IssuesPatchIssueResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.issues.patch_issue(
            id="9814a1fa-f0c6-408b-bf85-51dc3bc71ac7",
        )
        """
        _response = self._raw_client.patch_issue(
            id=id,
            assigned_to=assigned_to,
            due_date=due_date,
            external_ids=external_ids,
            status=status,
            request_options=request_options,
        )
        return _response.data

    def get_issues_stream(
        self,
        *,
        start_time: str,
        end_time: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        status: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        asset_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        assigned_to_route_stop_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> IssuesGetIssuesStreamResponseBody:
        """
        Returns all issues data that has been created or modified for your organization based on the time parameters passed in. Results are paginated and are sorted by last modified date. If you include an endTime, the endpoint will return data up until that point (exclusive). If you don’t include an endTime, you can continue to poll the API real-time with the pagination cursor that gets returned on every call.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Issues** under the Forms category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. Value is compared against `updatedAtTime`. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
            An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. Value is compared against `updatedAtTime`. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        status : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list containing status values to filter issues on. Valid values: `open`, `inProgress`, `resolved`, `dismissed`

        asset_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list containing up to 50 asset IDs to filter issues on. Issues with untracked assets can also be included by passing the value: 'untracked'.

        include : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma separated list of additional fields to include on requested objects. Valid values: `externalIds`

        assigned_to_route_stop_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list containing up to 50 route stop IDs to filter data on.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        IssuesGetIssuesStreamResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.issues.get_issues_stream(
            start_time="startTime",
        )
        """
        _response = self._raw_client.get_issues_stream(
            start_time=start_time,
            end_time=end_time,
            after=after,
            status=status,
            asset_ids=asset_ids,
            include=include,
            assigned_to_route_stop_ids=assigned_to_route_stop_ids,
            request_options=request_options,
        )
        return _response.data


class AsyncIssuesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawIssuesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawIssuesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawIssuesClient
        """
        return self._raw_client

    async def get_issues(
        self,
        *,
        ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> IssuesGetIssuesResponseBody:
        """
        Returns all issues data for the specified IDs.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Issues** under the Forms category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list containing up to 100 issue IDs to filter on. Can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the issue.

        include : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma separated list of additional fields to include on requested objects. Valid values: `externalIds`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        IssuesGetIssuesResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.issues.get_issues()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_issues(ids=ids, include=include, request_options=request_options)
        return _response.data

    async def patch_issue(
        self,
        *,
        id: str,
        assigned_to: typing.Optional[PatchIssueRequestBodyAssignedToRequestBody] = OMIT,
        due_date: typing.Optional[dt.datetime] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        status: typing.Optional[IssuesPatchIssueRequestBodyStatus] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> IssuesPatchIssueResponseBody:
        """
        Updates an instance of an issue.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Issues** under the Forms category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            ID of the issue. Can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the issue.

        assigned_to : typing.Optional[PatchIssueRequestBodyAssignedToRequestBody]

        due_date : typing.Optional[dt.datetime]
            Due date of the issue. UTC timestamp in RFC 3339 format.

        external_ids : typing.Optional[typing.Dict[str, str]]
            A map of external ids

        status : typing.Optional[IssuesPatchIssueRequestBodyStatus]
            Status of the issue.  Valid values: `open`, `inProgress`, `resolved`, `dismissed`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        IssuesPatchIssueResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.issues.patch_issue(
                id="9814a1fa-f0c6-408b-bf85-51dc3bc71ac7",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.patch_issue(
            id=id,
            assigned_to=assigned_to,
            due_date=due_date,
            external_ids=external_ids,
            status=status,
            request_options=request_options,
        )
        return _response.data

    async def get_issues_stream(
        self,
        *,
        start_time: str,
        end_time: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        status: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        asset_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        include: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        assigned_to_route_stop_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> IssuesGetIssuesStreamResponseBody:
        """
        Returns all issues data that has been created or modified for your organization based on the time parameters passed in. Results are paginated and are sorted by last modified date. If you include an endTime, the endpoint will return data up until that point (exclusive). If you don’t include an endTime, you can continue to poll the API real-time with the pagination cursor that gets returned on every call.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Issues** under the Forms category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. Value is compared against `updatedAtTime`. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
            An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. Value is compared against `updatedAtTime`. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        status : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list containing status values to filter issues on. Valid values: `open`, `inProgress`, `resolved`, `dismissed`

        asset_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list containing up to 50 asset IDs to filter issues on. Issues with untracked assets can also be included by passing the value: 'untracked'.

        include : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma separated list of additional fields to include on requested objects. Valid values: `externalIds`

        assigned_to_route_stop_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list containing up to 50 route stop IDs to filter data on.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        IssuesGetIssuesStreamResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.issues.get_issues_stream(
                start_time="startTime",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_issues_stream(
            start_time=start_time,
            end_time=end_time,
            after=after,
            status=status,
            asset_ids=asset_ids,
            include=include,
            assigned_to_route_stop_ids=assigned_to_route_stop_ids,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

TrailerStatReeferDoorStateZone1WithDecorationsTypeResponseBodyValue = typing.Union[
    typing.Literal["open", "closed"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class SafetyEventV2ContextLabelsResponseBody(UniversalBaseModel):
    """
    The most up-to-date context labels associated with the safety event. AI generated labels can be updated by Safety Admins.
    """

    author_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="authorId"),
        pydantic.Field(
            alias="authorId",
            description="The user ID associated with the user who created the context label. A value of 0 indicates the label is auto-generated.",
        ),
    ]
    created_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(alias="createdAtTime", description="Time the context label was created."),
    ]
    name: str = pydantic.Field()
    """
    Name of the context label
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .forms_check_boxes_value_object_response_body import FormsCheckBoxesValueObjectResponseBody
from .forms_date_time_value_object_response_body import FormsDateTimeValueObjectResponseBody
from .forms_media_record_object_response_body import FormsMediaRecordObjectResponseBody
from .forms_multiple_choice_value_object_response_body import FormsMultipleChoiceValueObjectResponseBody
from .forms_number_value_object_response_body import FormsNumberValueObjectResponseBody
from .forms_signature_value_object_response_body import FormsSignatureValueObjectResponseBody
from .forms_table_value_object_response_body import FormsTableValueObjectResponseBody
from .forms_text_value_object_response_body import FormsTextValueObjectResponseBody
from .qualification_field_input_object_response_body_type import QualificationFieldInputObjectResponseBodyType


class QualificationFieldInputObjectResponseBody(UniversalBaseModel):
    """
    Qualification input field object.
    """

    check_boxes_value: typing_extensions.Annotated[
        typing.Optional[FormsCheckBoxesValueObjectResponseBody],
        FieldMetadata(alias="checkBoxesValue"),
        pydantic.Field(alias="checkBoxesValue"),
    ] = None
    date_time_value: typing_extensions.Annotated[
        typing.Optional[FormsDateTimeValueObjectResponseBody],
        FieldMetadata(alias="dateTimeValue"),
        pydantic.Field(alias="dateTimeValue"),
    ] = None
    id: str = pydantic.Field()
    """
    ID of the qualification input field object.
    """

    label: typing.Optional[str] = pydantic.Field(default=None)
    """
    Qualification input field label.
    """

    media_list: typing_extensions.Annotated[
        typing.Optional[typing.List[FormsMediaRecordObjectResponseBody]],
        FieldMetadata(alias="mediaList"),
        pydantic.Field(alias="mediaList", description="List of qualification media record objects."),
    ] = None
    multiple_choice_value: typing_extensions.Annotated[
        typing.Optional[FormsMultipleChoiceValueObjectResponseBody],
        FieldMetadata(alias="multipleChoiceValue"),
        pydantic.Field(alias="multipleChoiceValue"),
    ] = None
    number_value: typing_extensions.Annotated[
        typing.Optional[FormsNumberValueObjectResponseBody],
        FieldMetadata(alias="numberValue"),
        pydantic.Field(alias="numberValue"),
    ] = None
    signature_value: typing_extensions.Annotated[
        typing.Optional[FormsSignatureValueObjectResponseBody],
        FieldMetadata(alias="signatureValue"),
        pydantic.Field(alias="signatureValue"),
    ] = None
    table_value: typing_extensions.Annotated[
        typing.Optional[FormsTableValueObjectResponseBody],
        FieldMetadata(alias="tableValue"),
        pydantic.Field(alias="tableValue"),
    ] = None
    text_value: typing_extensions.Annotated[
        typing.Optional[FormsTextValueObjectResponseBody],
        FieldMetadata(alias="textValue"),
        pydantic.Field(alias="textValue"),
    ] = None
    type: QualificationFieldInputObjectResponseBodyType = pydantic.Field()
    """
    Type of the qualification field.  Valid values: `number`, `text`, `multiple_choice`, `check_boxes`, `datetime`, `signature`, `media`, `table`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .driver_eld_ruleset_cycle import DriverEldRulesetCycle
from .driver_eld_ruleset_jurisdiction import DriverEldRulesetJurisdiction
from .driver_eld_ruleset_rest_break import DriverEldRulesetRestBreak
from .driver_eld_ruleset_restart import DriverEldRulesetRestart
from .driver_eld_ruleset_shift import DriverEldRulesetShift


class DriverEldRuleset(UniversalBaseModel):
    """
    An ELD ruleset for a driver.
    """

    break_: typing_extensions.Annotated[
        typing.Optional[DriverEldRulesetRestBreak], FieldMetadata(alias="break"), pydantic.Field(alias="break")
    ] = None
    cycle: typing.Optional[DriverEldRulesetCycle] = None
    jurisdiction: typing.Optional[DriverEldRulesetJurisdiction] = None
    restart: typing.Optional[DriverEldRulesetRestart] = None
    shift: typing.Optional[DriverEldRulesetShift] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .driver_ruleset_object_response_body_break import DriverRulesetObjectResponseBodyBreak
from .driver_ruleset_object_response_body_cycle import DriverRulesetObjectResponseBodyCycle
from .driver_ruleset_object_response_body_restart import DriverRulesetObjectResponseBodyRestart
from .driver_ruleset_object_response_body_shift import DriverRulesetObjectResponseBodyShift


class DriverRulesetObjectResponseBody(UniversalBaseModel):
    break_: typing_extensions.Annotated[
        typing.Optional[DriverRulesetObjectResponseBodyBreak],
        FieldMetadata(alias="break"),
        pydantic.Field(
            alias="break",
            description="The rest break required setting of the ELD ruleset applied to this driver.  Valid values: `Property (off-duty/sleeper)`, `Explosives/HazMat (on-duty)`",
        ),
    ] = None
    cycle: typing.Optional[DriverRulesetObjectResponseBodyCycle] = pydantic.Field(default=None)
    """
    The cycle of the ELD ruleset applied to this driver.  Valid values: `USA 60 hour / 7 day`, `USA 70 hour / 8 day`, `AK 80 hour / 8 day`, `AK 70 hour / 7 day`, `CA 80 hour / 8 day`, `CA 112 hour / 8 day`, `FL 80 hour / 8 day`, `FL 70 hour / 7 day`, `NE 80 hour / 8 day`, `NE 70 hour / 7 day`, `NC 80 hour / 8 day`, `NC 70 hour / 7 day`, `OK 70 hour / 8 day`, `OK 60 hour / 7 day`, `OR 80 hour / 8 day`, `OR 70 hour / 7 day`, `SC 80 hour / 8 day`, `SC 70 hour / 7 day`, `TX 70 hour / 7 day`, `WI 80 hour / 8 day`, `WI 70 hour / 7 day`, `Canada South Cycle 1 (70 hour / 7 day)`, `Canada South Cycle 2 (120 hour / 14 day)`, `Canada North Cycle 1 (80 hour / 7 day)`, `Canada North Cycle 2 (120 hour / 14 day)`
    """

    jurisdiction: typing.Optional[str] = pydantic.Field(default=None)
    """
    The jurisdiction of the ELD ruleset applied to this driver. These are specified by either `CS` or `CN` for Canada South and Canada North, respectively, or the ISO 3166-2 postal code for the supported state or territory.
    """

    restart: typing.Optional[DriverRulesetObjectResponseBodyRestart] = pydantic.Field(default=None)
    """
    The restart of the ELD ruleset applied to this driver.  Valid values: `34-hour Restart`, `24-hour Restart`, `36-hour Restart`, `72-hour Restart`
    """

    shift: typing.Optional[DriverRulesetObjectResponseBodyShift] = pydantic.Field(default=None)
    """
    The shift of the ELD ruleset applied to this driver.  Valid values: `US Interstate Property`, `US Interstate Passenger`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

PostFuelPurchaseRequestBodyPriceRequestBodyCurrency = typing.Union[
    typing.Literal["usd", "gbp", "cad", "eur", "chf", "mxn"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .api_error import ApiError
    from .client_wrapper import AsyncClientWrapper, BaseClientWrapper, SyncClientWrapper
    from .datetime_utils import serialize_datetime
    from .file import File, convert_file_dict_to_httpx_tuples, with_content_type
    from .http_client import AsyncHttpClient, HttpClient
    from .http_response import AsyncHttpResponse, HttpResponse
    from .jsonable_encoder import jsonable_encoder
    from .pagination import AsyncPager, SyncPager
    from .pydantic_utilities import (
        IS_PYDANTIC_V2,
        UniversalBaseModel,
        UniversalRootModel,
        parse_obj_as,
        universal_field_validator,
        universal_root_validator,
        update_forward_refs,
    )
    from .query_encoder import encode_query
    from .remove_none_from_dict import remove_none_from_dict
    from .request_options import RequestOptions
    from .serialization import FieldMetadata, convert_and_respect_annotation_metadata
_dynamic_imports: typing.Dict[str, str] = {
    "ApiError": ".api_error",
    "AsyncClientWrapper": ".client_wrapper",
    "AsyncHttpClient": ".http_client",
    "AsyncHttpResponse": ".http_response",
    "AsyncPager": ".pagination",
    "BaseClientWrapper": ".client_wrapper",
    "FieldMetadata": ".serialization",
    "File": ".file",
    "HttpClient": ".http_client",
    "HttpResponse": ".http_response",
    "IS_PYDANTIC_V2": ".pydantic_utilities",
    "RequestOptions": ".request_options",
    "SyncClientWrapper": ".client_wrapper",
    "SyncPager": ".pagination",
    "UniversalBaseModel": ".pydantic_utilities",
    "UniversalRootModel": ".pydantic_utilities",
    "convert_and_respect_annotation_metadata": ".serialization",
    "convert_file_dict_to_httpx_tuples": ".file",
    "encode_query": ".query_encoder",
    "jsonable_encoder": ".jsonable_encoder",
    "parse_obj_as": ".pydantic_utilities",
    "remove_none_from_dict": ".remove_none_from_dict",
    "serialize_datetime": ".datetime_utils",
    "universal_field_validator": ".pydantic_utilities",
    "universal_root_validator": ".pydantic_utilities",
    "update_forward_refs": ".pydantic_utilities",
    "with_content_type": ".file",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "ApiError",
    "AsyncClientWrapper",
    "AsyncHttpClient",
    "AsyncHttpResponse",
    "AsyncPager",
    "BaseClientWrapper",
    "FieldMetadata",
    "File",
    "HttpClient",
    "HttpResponse",
    "IS_PYDANTIC_V2",
    "RequestOptions",
    "SyncClientWrapper",
    "SyncPager",
    "UniversalBaseModel",
    "UniversalRootModel",
    "convert_and_respect_annotation_metadata",
    "convert_file_dict_to_httpx_tuples",
    "encode_query",
    "jsonable_encoder",
    "parse_obj_as",
    "remove_none_from_dict",
    "serialize_datetime",
    "universal_field_validator",
    "universal_root_validator",
    "update_forward_refs",
    "with_content_type",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .time_range_object_request_body_days_of_week_item import TimeRangeObjectRequestBodyDaysOfWeekItem


class TimeRangeObjectRequestBody(UniversalBaseModel):
    """
    A daily time range. If start time of day is greater than end time of day, then the time range applies overnight from the specified day of week into the following day.
    """

    days_of_week: typing_extensions.Annotated[
        typing.List[TimeRangeObjectRequestBodyDaysOfWeekItem],
        FieldMetadata(alias="daysOfWeek"),
        pydantic.Field(alias="daysOfWeek", description="Which days this timezone applies to."),
    ]
    end_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="endTime"),
        pydantic.Field(
            alias="endTime",
            description="The time of day at which the time range starts. In 24 hour kitchen clock format.",
        ),
    ]
    start_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="startTime"),
        pydantic.Field(
            alias="startTime",
            description="The time of day at which the time range starts. In 24 hour kitchen clock format.",
        ),
    ]
    timezone: str = pydantic.Field()
    """
    The timezone of the time range uses [IANA timezone database](https://www.iana.org/time-zones) keys (e.g. `America/Los_Angeles`, `America/New_York`, `Europe/London`, etc.). You can find a mapping of common timezone formats to IANA timezone keys [here](https://unicode.org/cldr/charts/latest/supplemental/zone_tzid.html).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

SpecificVehicleFaultCodeObjectRequestBodyType = typing.Union[
    typing.Literal["INVALID_FAULT_CODE_TYPE", "J1939_DTC", "J1939_SPN", "PASSENGER_DTC"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .dvir_resolved_by_object_response_body import DvirResolvedByObjectResponseBody
from .goa_trailer_tiny_response_response_body import GoaTrailerTinyResponseResponseBody
from .vehicle_with_gateway_tiny_response_response_body import VehicleWithGatewayTinyResponseResponseBody


class DvirDefectsObjectV20220913ResponseBody(UniversalBaseModel):
    """
    A description of a DVIR defect
    """

    comment: typing.Optional[str] = pydantic.Field(default=None)
    """
    Comment on the defect.
    """

    created_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(
            alias="createdAtTime", description="Time when the defect was created. UTC timestamp in RFC 3339 format."
        ),
    ]
    defect_type: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="defectType"),
        pydantic.Field(alias="defectType", description="The type of DVIR defect."),
    ]
    id: str = pydantic.Field()
    """
    The ID of the defect.
    """

    is_resolved: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="isResolved"),
        pydantic.Field(alias="isResolved", description="Signifies if this defect is resolved."),
    ]
    mechanic_notes: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="mechanicNotes"),
        pydantic.Field(alias="mechanicNotes", description="The mechanic notes on this defect."),
    ] = None
    mechanic_notes_updated_at_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="mechanicNotesUpdatedAtTime"),
        pydantic.Field(
            alias="mechanicNotesUpdatedAtTime",
            description="Time when mechanic notes were last updated. UTC timestamp in RFC 3339 format.",
        ),
    ] = None
    resolved_at_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="resolvedAtTime"),
        pydantic.Field(
            alias="resolvedAtTime",
            description="Time when this defect was resolved. Will not be returned if the defect is unresolved. UTC timestamp in RFC 3339 format.",
        ),
    ] = None
    resolved_by: typing_extensions.Annotated[
        typing.Optional[DvirResolvedByObjectResponseBody],
        FieldMetadata(alias="resolvedBy"),
        pydantic.Field(alias="resolvedBy"),
    ] = None
    trailer: typing.Optional[GoaTrailerTinyResponseResponseBody] = None
    vehicle: typing.Optional[VehicleWithGatewayTinyResponseResponseBody] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
